# -*- coding: utf-8 -*-
"""
Interaction Tests
~~~~~~~~~~~~~~~~~
"""
import argh


def parse_choice(choice, **kwargs):
    argh.io._input = lambda prompt: choice
    return argh.confirm('test', **kwargs)


def test_simple():
    assert None == parse_choice('')
    assert None == parse_choice('', default=None)
    assert True == parse_choice('', default=True)
    assert False == parse_choice('', default=False)

    assert True == parse_choice('y')
    assert True == parse_choice('y', default=True)
    assert True == parse_choice('y', default=False)
    assert True == parse_choice('y', default=None)

    assert False == parse_choice('n')
    assert False == parse_choice('n', default=True)
    assert False == parse_choice('n', default=False)
    assert False == parse_choice('n', default=None)

    assert None == parse_choice('x') == None


def test_prompt():
    "Prompt is properly formatted"
    prompts = []

    def raw_input_mock(prompt):
        prompts.append(prompt)
    argh.io._input = raw_input_mock

    argh.confirm('do smth')
    assert prompts[-1] == 'do smth? (y/n)'

    argh.confirm('do smth', default=None)
    assert prompts[-1] == 'do smth? (y/n)'

    argh.confirm('do smth', default=True)
    assert prompts[-1] == 'do smth? (Y/n)'

    argh.confirm('do smth', default=False)
    assert prompts[-1] == 'do smth? (y/N)'


def test_encoding():
    "Unicode and bytes are accepted as prompt message"
    def raw_input_mock(prompt):
        if not argh.six.PY3:
            assert isinstance(prompt, argh.six.binary_type)
    argh.io._input = raw_input_mock
    argh.confirm(argh.six.u('привет'))



# -*- coding: utf-8 -*-
#
#  Copyright (c) 2010—2012 Andrey Mikhailenko and contributors
#
#  This file is part of Argh.
#
#  Argh is free software under terms of the GNU Lesser
#  General Public License version 3 (LGPLv3) as published by the Free
#  Software Foundation. See the file README for copying conditions.
#
"""
Output Processing
~~~~~~~~~~~~~~~~~
"""
import locale

from argh.six import binary_type, text_type, PY3


__all__ = ['dump', 'encode_output', 'safe_input']


def _input(prompt):
    # this function can be mocked up in tests
    if PY3:
        return input(prompt)
    else:
        return raw_input(prompt)


def safe_input(prompt):
    "Prompts user for input. Correctly handles prompt message encoding."

    if PY3:
        if not isinstance(prompt, text_type):
            # Python 3.x: bytes →  unicode
            prompt = prompt.decode()
    else:
        if isinstance(prompt, text_type):
            # Python 2.x: unicode →  bytes
            encoding = locale.getpreferredencoding() or 'utf-8'
            prompt = prompt.encode(encoding)

    return _input(prompt)


def encode_output(value, output_file):
    """ Encodes given value so it can be written to given file object.

    Value may be Unicode, binary string or any other data type.

    The exact behaviour depends on the Python version:

    Python 3.x

        `sys.stdout` is a `_io.TextIOWrapper` instance that accepts `str`
        (unicode) and breaks on `bytes`.

        It is OK to simply assume that everything is Unicode unless special
        handling is introduced in the client code.

        Thus, no additional processing is performed.

    Python 2.x

        `sys.stdout` is a file-like object that accepts `str` (bytes)
        and breaks when `unicode` is passed to `sys.stdout.write()`.

        We can expect both Unicode and bytes. They need to be encoded so as
        to match the file object encoding.

        The output is binary if the object is a TTY; otherwise it's Unicode.

    """
    if not PY3:
        # handle special cases in Python 2.x

        if output_file.isatty():
            # TTY expects bytes
            if isinstance(value, text_type):
                encoding = getattr(output_file, 'encoding', None)
                encoding = encoding or locale.getpreferredencoding() or 'utf-8'
                # unicode →  binary
                return value.encode(encoding)
            return binary_type(value)

        if isinstance(value, binary_type):
            # binary →  unicode
            return value.decode('utf-8')

    # whatever → unicode
    return text_type(value)


def dump(raw_data, output_file):
    """ Writes given line to given output file.
    See :func:`encode_output` for details.
    """
    data = encode_output(raw_data, output_file)
    output_file.write(data)



# -*- coding: utf-8 -*-
import mock

import argh


# just unit/mock tests
def test_mock_checkout():
    p = argh.ArghParser()
    p.add_argument = mock.MagicMock()

    @argh.arg('foo', default=1, nargs='+')
    def cmd(args):
        pass

    argh.set_default_command(p, cmd)
    p.add_argument.assert_called_with('foo', default=1, nargs='+', type=int)
